/*
 * K3DObject.h
 *
 *  Created on: Sep 28, 2008
 *      Author: Santi Ontann
 */

#ifndef K3DOBJECT_H_
#define K3DOBJECT_H_

#include "KObject.h"

#include "GLTManager.h"

#include "quaternion.h"

class K3DObject : public KObject
{
    public:
    	K3DObject(int x, int y, int z, int sfx_volume);
        virtual ~K3DObject();

        virtual void draw(float angle,GLTManager *GLTM);
        virtual void draw(float angle,int x_offset,int y_offset,GLTManager *GLTM);

        virtual bool is_a(Symbol *c);
        virtual bool is_a(char *c);

        void set_z(float z) {
        	m_z = z;
        }

        void drawShadows(int x_offs,int y_offs,float lvx,float lvy,float lvz,float depth);

        void createCube(int dx,int dy,int dz,float r,float g,float b,float a);

        void createPlane(int dx,int dy,float r,float g,float b,float a);

    protected:

    	void drawInternal(int xoffs,int yoffs);
    	void drawInternalCartoon(int xoffs,int yoffs);

    	void setFacePointColor(int f,int p);
    	void computeNormals(void);
    	void fixNormals(float x,float y, float z);	// makes all the normals point away from (x,y,z) (the center of the object)
    	void computeEdges(float maxAngle);


    	float m_r,m_g,m_b,m_a;	// used if no face or vertex color specified
    	float m_z;
    	Quaternion m_d;

    	int m_nfaces;
    	int m_npoints;
    	int m_nedges;
    	float *m_points;
    	int *m_faces;
    	int *m_edges;
    	float *m_normals;
    	float *m_face_colors;	// (optionsl)
    	float *m_point_colors;	// (optionsl)

    	/* Shadow information: */
    	int *m_adjacent_faces;

    	/* Sub objects: */
    	List<K3DObject> m_parts;
	
		/* Fake "last_tile_used" for collision purposes: */
		GLTile *collisionTile;

};

#endif /* K3DOBJECT_H_ */
